searchData={"content_type":"text/markdown","items":[{"doc":"Configuration structure for using reactive commons abstraction\n\nThe required properties are:\n- application_name: \"my_app\"\n\nOptionally you can change the default values for another property\n```elixir\n%{\n  reply_exchange: \"globalReply\",\n  direct_exchange: \"directMessages\",\n  events_exchange: \"domainEvents\",\n  connection_props: \"amqp://guest:guest@localhost\", this value is passed as uri_or_options in https://hexdocs.pm/amqp/AMQP.Connection.html#open/1\n  connection_assignation: %{\n    ReplyListener: ListenerConn,\n    QueryListener: ListenerConn,\n    CommandListener: ListenerConn,\n    EventListener: ListenerConn,\n    NotificationEventListener: ListenerConn,\n    MessageExtractor: ListenerConn,\n    MessageSender: SenderConn,\n    ListenerController: SenderConn,\n  },\n  queries_reply: true,\n  with_dlq_retry: false,\n  retry_delay: 500,\n  max_retries: 10,\n  prefetch_count: 250,\n}\n```","ref":"AsyncConfig.html","title":"AsyncConfig","type":"module"},{"doc":"Creates a new default valued AsyncConfig struct with the specified app name.","ref":"AsyncConfig.html#new/1","title":"AsyncConfig.new/1","type":"function"},{"doc":"Queries represent a intention for getting information about something, that query must to be processed by the domain\ncontext with that responsibility and that context must respond with the information requested throught request/reply\npattern. An example of a query may be: \"UserInfo\".","ref":"AsyncQuery.html","title":"AsyncQuery","type":"module"},{"doc":"Creates a new AsyncQuery structure","ref":"AsyncQuery.html#new/2","title":"AsyncQuery.new/2","type":"function"},{"doc":"iex> AsyncQuery.new(\"UserInfo\", %{name: \"username\"})\n    %AsyncQuery{queryData: %{name: \"username\"}, resource: \"UserInfo\"}","ref":"AsyncQuery.html#new/2-examples","title":"Examples - AsyncQuery.new/2","type":"function"},{"doc":"Commands represent a intention for doing something, that intention must to be done by the domain context with that\nresponsibility. An example of a command may be: \"RegisterUser\" or \"SendNotification\".","ref":"Command.html","title":"Command","type":"module"},{"doc":"Creates a new Command structure with a generated command_id","ref":"Command.html#new/2","title":"Command.new/2","type":"function"},{"doc":"iex> Command.new(\"RegisterUser\", %{name: \"username\", email: \"user@example.com\"})\n    %Command{\n      commandId: \"a313623c-4701-40d3-95e0-bde25e02b52a\",\n      data: %{email: \"user@example.com\", name: \"username\"},\n      name: \"registerUser\"\n    }","ref":"Command.html#new/2-examples","title":"Examples - Command.new/2","type":"function"},{"doc":"Creates a new Command structure","ref":"Command.html#new/3","title":"Command.new/3","type":"function"},{"doc":"iex> Command.new(\"RegisterUser\", %{name: \"username\", email: \"user@example.com\"}, \"4016ab59-16d0-41cd-ba4f-52ce200f8ea8\")\n    %Command{\n      commandId: \"4016ab59-16d0-41cd-ba4f-52ce200f8ea8\",\n      data: %{email: \"user@example.com\", name: \"username\"},\n      name: \"registerUser\"\n    }","ref":"Command.html#new/3-examples","title":"Examples - Command.new/3","type":"function"},{"doc":"","ref":"CommandExecutor.html","title":"CommandExecutor","type":"module"},{"doc":"","ref":"CommandExecutor.html#decode/1","title":"CommandExecutor.decode/1","type":"function"},{"doc":"","ref":"CommandExecutor.html#get_handler_path/2","title":"CommandExecutor.get_handler_path/2","type":"function"},{"doc":"","ref":"CommandExecutor.html#handle_message/1","title":"CommandExecutor.handle_message/1","type":"function"},{"doc":"","ref":"CommandExecutor.html#on_post_process/2","title":"CommandExecutor.on_post_process/2","type":"function"},{"doc":"","ref":"CommandListener.html","title":"CommandListener","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"CommandListener.html#child_spec/1","title":"CommandListener.child_spec/1","type":"function"},{"doc":"","ref":"CommandListener.html#consume/3","title":"CommandListener.consume/3","type":"function"},{"doc":"","ref":"CommandListener.html#get_handlers/0","title":"CommandListener.get_handlers/0","type":"function"},{"doc":"","ref":"CommandListener.html#start_link/1","title":"CommandListener.start_link/1","type":"function"},{"doc":"","ref":"ConnectionsHolder.html","title":"ConnectionsHolder","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ConnectionsHolder.html#child_spec/1","title":"ConnectionsHolder.child_spec/1","type":"function"},{"doc":"","ref":"ConnectionsHolder.html#get_connection_async/1","title":"ConnectionsHolder.get_connection_async/1","type":"function"},{"doc":"","ref":"ConnectionsHolder.html#start_link/1","title":"ConnectionsHolder.start_link/1","type":"function"},{"doc":"This module allows the commands emission and async queries requests.","ref":"DirectAsyncGateway.html","title":"DirectAsyncGateway","type":"module"},{"doc":"","ref":"DirectAsyncGateway.html#headers/0","title":"DirectAsyncGateway.headers/0","type":"function"},{"doc":"","ref":"DirectAsyncGateway.html#headers/2","title":"DirectAsyncGateway.headers/2","type":"function"},{"doc":"","ref":"DirectAsyncGateway.html#request_reply/2","title":"DirectAsyncGateway.request_reply/2","type":"function"},{"doc":"","ref":"DirectAsyncGateway.html#request_reply_wait/2","title":"DirectAsyncGateway.request_reply_wait/2","type":"function"},{"doc":"","ref":"DirectAsyncGateway.html#send_command/2","title":"DirectAsyncGateway.send_command/2","type":"function"},{"doc":"","ref":"DirectAsyncGateway.html#wait_reply/2","title":"DirectAsyncGateway.wait_reply/2","type":"function"},{"doc":"","ref":"DiscardNotifier.html","title":"DiscardNotifier","type":"module"},{"doc":"","ref":"DiscardNotifier.html#notify/1","title":"DiscardNotifier.notify/1","type":"function"},{"doc":"Events represent a fact inside the domain, it is the representation of a decision or a state change that a system want\nto notify to its subscribers. Events represents facts that nobody can change, so events are not intentions or requests\nof anything, An example may be and UserRegistered or a NotificationSent.\n\nEvents are the most important topic in a Publish-Subscribe system, because this element let’s notify a many\nstakeholders in a specific event. An other benefit is the system is decouple, because you can add more subscriber to\nthe system without modify some component.","ref":"DomainEvent.html","title":"DomainEvent","type":"module"},{"doc":"Creates a new DomainEvent structure with a generated event_id","ref":"DomainEvent.html#new/2","title":"DomainEvent.new/2","type":"function"},{"doc":"iex> DomainEvent.new(\"UserRegistered\", %{name: \"username\", email: \"user@example.com\", createdAt: \"2021-05-11T15:00:47.380Z\"})\n    %DomainEvent{\n      data: %{\n        createdAt: \"2021-05-11T15:00:47.380Z\",\n        email: \"user@example.com\",\n        name: \"username\"\n      },\n      eventId: \"852e6f59-f920-45e0-bce8-75f1e74647ff\",\n      name: \"UserRegistered\"\n    }","ref":"DomainEvent.html#new/2-examples","title":"Examples - DomainEvent.new/2","type":"function"},{"doc":"Creates a new DomainEvent structure","ref":"DomainEvent.html#new/3","title":"DomainEvent.new/3","type":"function"},{"doc":"iex> DomainEvent.new(\"UserRegistered\", %{name: \"username\", email: \"user@example.com\", createdAt: \"2021-05-11T15:00:47.380Z\"}, \"852e6f59-f920-45e0-bce8-75f1e74647aa\")\n    %DomainEvent{\n      data: %{\n        createdAt: \"2021-05-11T15:00:47.380Z\",\n        email: \"user@example.com\",\n        name: \"username\"\n      },\n      eventId: \"852e6f59-f920-45e0-bce8-75f1e74647aa\",\n      name: \"UserRegistered\"\n    }","ref":"DomainEvent.html#new/3-examples","title":"Examples - DomainEvent.new/3","type":"function"},{"doc":"This module allows the domain events emission.","ref":"DomainEventBus.html","title":"DomainEventBus","type":"module"},{"doc":"","ref":"DomainEventBus.html#emit/1","title":"DomainEventBus.emit/1","type":"function"},{"doc":"","ref":"DomainEventBus.html#headers/0","title":"DomainEventBus.headers/0","type":"function"},{"doc":"","ref":"EventExecutor.html","title":"EventExecutor","type":"module"},{"doc":"","ref":"EventExecutor.html#decode/1","title":"EventExecutor.decode/1","type":"function"},{"doc":"","ref":"EventExecutor.html#get_handler_path/2","title":"EventExecutor.get_handler_path/2","type":"function"},{"doc":"","ref":"EventExecutor.html#handle_message/1","title":"EventExecutor.handle_message/1","type":"function"},{"doc":"","ref":"EventExecutor.html#on_post_process/2","title":"EventExecutor.on_post_process/2","type":"function"},{"doc":"","ref":"EventListener.html","title":"EventListener","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"EventListener.html#child_spec/1","title":"EventListener.child_spec/1","type":"function"},{"doc":"","ref":"EventListener.html#consume/3","title":"EventListener.consume/3","type":"function"},{"doc":"","ref":"EventListener.html#get_handlers/0","title":"EventListener.get_handlers/0","type":"function"},{"doc":"","ref":"EventListener.html#start_link/1","title":"EventListener.start_link/1","type":"function"},{"doc":"Implements generic behaviour for message executors","ref":"GenericExecutor.html","title":"GenericExecutor","type":"behaviour"},{"doc":"","ref":"GenericExecutor.html#calc_duration/1","title":"GenericExecutor.calc_duration/1","type":"function"},{"doc":"Decode message payload from MessageToHandle.","ref":"GenericExecutor.html#c:decode/1","title":"GenericExecutor.decode/1","type":"callback"},{"doc":"Extract handler function path (name) from the message.","ref":"GenericExecutor.html#c:get_handler_path/2","title":"GenericExecutor.get_handler_path/2","type":"callback"},{"doc":"It's called when handler return (optional).","ref":"GenericExecutor.html#c:on_post_process/2","title":"GenericExecutor.on_post_process/2","type":"callback"},{"doc":"","ref":"GenericExecutor.html#report_to_telemetry/4","title":"GenericExecutor.report_to_telemetry/4","type":"function"},{"doc":"","ref":"GenericExecutor.html#requeue_or_ack/3","title":"GenericExecutor.requeue_or_ack/3","type":"function"},{"doc":"","ref":"GenericExecutor.html#t:handler_response/0","title":"GenericExecutor.handler_response/0","type":"type"},{"doc":"","ref":"GenericExecutor.html#t:parsed_payload/0","title":"GenericExecutor.parsed_payload/0","type":"type"},{"doc":"Implements generic behaviour for event listeners","ref":"GenericListener.html","title":"GenericListener","type":"behaviour"},{"doc":"Create resource topology.","ref":"GenericListener.html#c:create_topology/1","title":"GenericListener.create_topology/1","type":"callback"},{"doc":"","ref":"GenericListener.html#declare_dlq/4","title":"GenericListener.declare_dlq/4","type":"function"},{"doc":"","ref":"GenericListener.html#get_correlation_id/1","title":"GenericListener.get_correlation_id/1","type":"function"},{"doc":"Get initial state.","ref":"GenericListener.html#c:initial_state/0","title":"GenericListener.initial_state/0","type":"callback"},{"doc":"Evaluate if should listen for this kind of events.","ref":"GenericListener.html#c:should_listen/0","title":"GenericListener.should_listen/0","type":"callback"},{"doc":"This module allows the subscription for events, commands and async queries and for registering their respective\n  handlers","ref":"HandlerRegistry.html","title":"HandlerRegistry","type":"module"},{"doc":"","ref":"HandlerRegistry.html#commit_config/1","title":"HandlerRegistry.commit_config/1","type":"function"},{"doc":"","ref":"HandlerRegistry.html#handle_command/2","title":"HandlerRegistry.handle_command/2","type":"function"},{"doc":"","ref":"HandlerRegistry.html#handle_command/3","title":"HandlerRegistry.handle_command/3","type":"function"},{"doc":"","ref":"HandlerRegistry.html#listen_event/2","title":"HandlerRegistry.listen_event/2","type":"function"},{"doc":"","ref":"HandlerRegistry.html#listen_event/3","title":"HandlerRegistry.listen_event/3","type":"function"},{"doc":"","ref":"HandlerRegistry.html#listen_notification_event/2","title":"HandlerRegistry.listen_notification_event/2","type":"function"},{"doc":"","ref":"HandlerRegistry.html#listen_notification_event/3","title":"HandlerRegistry.listen_notification_event/3","type":"function"},{"doc":"","ref":"HandlerRegistry.html#serve_query/2","title":"HandlerRegistry.serve_query/2","type":"function"},{"doc":"","ref":"HandlerRegistry.html#serve_query/3","title":"HandlerRegistry.serve_query/3","type":"function"},{"doc":"","ref":"HandlersConfig.html","title":"HandlersConfig","type":"module"},{"doc":"","ref":"HandlersConfig.html#add_listener/4","title":"HandlersConfig.add_listener/4","type":"function"},{"doc":"","ref":"HandlersConfig.html#new/0","title":"HandlersConfig.new/0","type":"function"},{"doc":"","ref":"HeaderExtractor.html","title":"HeaderExtractor","type":"module"},{"doc":"","ref":"HeaderExtractor.html#get_header_value/3","title":"HeaderExtractor.get_header_value/3","type":"function"},{"doc":"","ref":"HeaderExtractor.html#get_x_death_count/1","title":"HeaderExtractor.get_x_death_count/1","type":"function"},{"doc":"","ref":"ListenerController.html","title":"ListenerController","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ListenerController.html#child_spec/1","title":"ListenerController.child_spec/1","type":"function"},{"doc":"","ref":"ListenerController.html#configure/1","title":"ListenerController.configure/1","type":"function"},{"doc":"","ref":"ListenerController.html#start_link/1","title":"ListenerController.start_link/1","type":"function"},{"doc":"","ref":"ListenersValidator.html","title":"ListenersValidator","type":"module"},{"doc":"","ref":"ListenersValidator.html#has_handlers/1","title":"ListenersValidator.has_handlers/1","type":"function"},{"doc":"","ref":"ListenersValidator.html#should_listen_replies/1","title":"ListenersValidator.should_listen_replies/1","type":"function"},{"doc":"This module maintain the required settings to achieve communication with the message broker, it also provides default semantic settings","ref":"MessageContext.html","title":"MessageContext","type":"module"},{"doc":"","ref":"MessageContext.html#application_name/0","title":"MessageContext.application_name/0","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"MessageContext.html#child_spec/1","title":"MessageContext.child_spec/1","type":"function"},{"doc":"","ref":"MessageContext.html#command_queue_name/0","title":"MessageContext.command_queue_name/0","type":"function"},{"doc":"","ref":"MessageContext.html#config/0","title":"MessageContext.config/0","type":"function"},{"doc":"","ref":"MessageContext.html#direct_exchange_name/0","title":"MessageContext.direct_exchange_name/0","type":"function"},{"doc":"","ref":"MessageContext.html#event_queue_name/0","title":"MessageContext.event_queue_name/0","type":"function"},{"doc":"","ref":"MessageContext.html#events_exchange_name/0","title":"MessageContext.events_exchange_name/0","type":"function"},{"doc":"","ref":"MessageContext.html#handlers/0","title":"MessageContext.handlers/0","type":"function"},{"doc":"","ref":"MessageContext.html#handlers_configured?/0","title":"MessageContext.handlers_configured?/0","type":"function"},{"doc":"","ref":"MessageContext.html#init/1","title":"MessageContext.init/1","type":"function"},{"doc":"","ref":"MessageContext.html#max_retries/0","title":"MessageContext.max_retries/0","type":"function"},{"doc":"","ref":"MessageContext.html#notification_event_queue_name/0","title":"MessageContext.notification_event_queue_name/0","type":"function"},{"doc":"","ref":"MessageContext.html#prefetch_count/0","title":"MessageContext.prefetch_count/0","type":"function"},{"doc":"","ref":"MessageContext.html#query_queue_name/0","title":"MessageContext.query_queue_name/0","type":"function"},{"doc":"","ref":"MessageContext.html#reply_exchange_name/0","title":"MessageContext.reply_exchange_name/0","type":"function"},{"doc":"","ref":"MessageContext.html#reply_queue_name/0","title":"MessageContext.reply_queue_name/0","type":"function"},{"doc":"","ref":"MessageContext.html#reply_routing_key/0","title":"MessageContext.reply_routing_key/0","type":"function"},{"doc":"","ref":"MessageContext.html#retry_delay/0","title":"MessageContext.retry_delay/0","type":"function"},{"doc":"","ref":"MessageContext.html#save_handlers_config/1","title":"MessageContext.save_handlers_config/1","type":"function"},{"doc":"","ref":"MessageContext.html#start_link/1","title":"MessageContext.start_link/1","type":"function"},{"doc":"","ref":"MessageContext.html#table/0","title":"MessageContext.table/0","type":"function"},{"doc":"","ref":"MessageContext.html#with_dlq_retry/0","title":"MessageContext.with_dlq_retry/0","type":"function"},{"doc":"","ref":"MessageExtractor.html","title":"MessageExtractor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"MessageExtractor.html#child_spec/1","title":"MessageExtractor.child_spec/1","type":"function"},{"doc":"","ref":"MessageExtractor.html#extract_message/3","title":"MessageExtractor.extract_message/3","type":"function"},{"doc":"","ref":"MessageExtractor.html#inspect_message/3","title":"MessageExtractor.inspect_message/3","type":"function"},{"doc":"","ref":"MessageExtractor.html#scan_messages/2","title":"MessageExtractor.scan_messages/2","type":"function"},{"doc":"","ref":"MessageExtractor.html#start_link/2","title":"MessageExtractor.start_link/2","type":"function"},{"doc":"","ref":"MessageHeaders.html","title":"MessageHeaders","type":"module"},{"doc":"","ref":"MessageHeaders.html#h_COMPLETION_ONLY_SIGNAL/0","title":"MessageHeaders.h_COMPLETION_ONLY_SIGNAL/0","type":"function"},{"doc":"","ref":"MessageHeaders.html#h_CORRELATION_ID/0","title":"MessageHeaders.h_CORRELATION_ID/0","type":"function"},{"doc":"","ref":"MessageHeaders.html#h_REPLY_ID/0","title":"MessageHeaders.h_REPLY_ID/0","type":"function"},{"doc":"","ref":"MessageHeaders.html#h_SERVED_QUERY_ID/0","title":"MessageHeaders.h_SERVED_QUERY_ID/0","type":"function"},{"doc":"","ref":"MessageHeaders.html#h_SIGNAL_TYPE/0","title":"MessageHeaders.h_SIGNAL_TYPE/0","type":"function"},{"doc":"","ref":"MessageHeaders.html#h_SOURCE_APPLICATION/0","title":"MessageHeaders.h_SOURCE_APPLICATION/0","type":"function"},{"doc":"","ref":"MessageHeaders.html#h_X_DEATH/0","title":"MessageHeaders.h_X_DEATH/0","type":"function"},{"doc":"","ref":"MessageHeaders.html#h_X_DEATH_COUNT/0","title":"MessageHeaders.h_X_DEATH_COUNT/0","type":"function"},{"doc":"This module initializes and supervises all required processes to enable the reactive commons ecosystem","ref":"MessageRuntime.html","title":"MessageRuntime","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"MessageRuntime.html#child_spec/1","title":"MessageRuntime.child_spec/1","type":"function"},{"doc":"","ref":"MessageRuntime.html#start_link/1","title":"MessageRuntime.start_link/1","type":"function"},{"doc":"","ref":"MessageSender.html","title":"MessageSender","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"MessageSender.html#child_spec/1","title":"MessageSender.child_spec/1","type":"function"},{"doc":"","ref":"MessageSender.html#send_message/1","title":"MessageSender.send_message/1","type":"function"},{"doc":"","ref":"MessageSender.html#start_link/1","title":"MessageSender.start_link/1","type":"function"},{"doc":"","ref":"MessageToHandle.html","title":"MessageToHandle","type":"module"},{"doc":"","ref":"MessageToHandle.html#new/4","title":"MessageToHandle.new/4","type":"function"},{"doc":"This module generates unique identifiers for temporary queue names, message ids and routing keys.","ref":"NameGenerator.html","title":"NameGenerator","type":"module"},{"doc":"Generate an hex unique random id.","ref":"NameGenerator.html#generate/0","title":"NameGenerator.generate/0","type":"function"},{"doc":"```elixir\niex> NameGenerator.generate()\n\"fb49a0ecd60c4d2092643b4cfe272106\"\n```","ref":"NameGenerator.html#generate/0-examples","title":"Examples - NameGenerator.generate/0","type":"function"},{"doc":"Generate a new id using UUID v4 with a fixed prefix.","ref":"NameGenerator.html#generate/2","title":"NameGenerator.generate/2","type":"function"},{"doc":"```elixir\niex> NameGenerator.generate(\"sample_app\")\n\"sample_app-fb49a0ecd60c4d2092643b4cfe272106\"\n```","ref":"NameGenerator.html#generate/2-examples","title":"Examples - NameGenerator.generate/2","type":"function"},{"doc":"Generate a new id using UUID v4.","ref":"NameGenerator.html#message_id/0","title":"NameGenerator.message_id/0","type":"function"},{"doc":"```elixir\niex> NameGenerator.message_id()\n\"fb49a0ec-d60c-4d20-9264-3b4cfe272106\"\n```","ref":"NameGenerator.html#message_id/0-examples","title":"Examples - NameGenerator.message_id/0","type":"function"},{"doc":"","ref":"NotificationEventExecutor.html","title":"NotificationEventExecutor","type":"module"},{"doc":"","ref":"NotificationEventExecutor.html#decode/1","title":"NotificationEventExecutor.decode/1","type":"function"},{"doc":"","ref":"NotificationEventExecutor.html#get_handler_path/2","title":"NotificationEventExecutor.get_handler_path/2","type":"function"},{"doc":"","ref":"NotificationEventExecutor.html#handle_message/1","title":"NotificationEventExecutor.handle_message/1","type":"function"},{"doc":"","ref":"NotificationEventExecutor.html#on_post_process/2","title":"NotificationEventExecutor.on_post_process/2","type":"function"},{"doc":"","ref":"NotificationEventListener.html","title":"NotificationEventListener","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NotificationEventListener.html#child_spec/1","title":"NotificationEventListener.child_spec/1","type":"function"},{"doc":"","ref":"NotificationEventListener.html#consume/3","title":"NotificationEventListener.consume/3","type":"function"},{"doc":"","ref":"NotificationEventListener.html#get_handlers/0","title":"NotificationEventListener.get_handlers/0","type":"function"},{"doc":"","ref":"NotificationEventListener.html#start_link/1","title":"NotificationEventListener.start_link/1","type":"function"},{"doc":"","ref":"OutMessage.html","title":"OutMessage","type":"module"},{"doc":"","ref":"OutMessage.html#new/1","title":"OutMessage.new/1","type":"function"},{"doc":"","ref":"QueryExecutor.html","title":"QueryExecutor","type":"module"},{"doc":"","ref":"QueryExecutor.html#handle_message/1","title":"QueryExecutor.handle_message/1","type":"function"},{"doc":"","ref":"QueryListener.html","title":"QueryListener","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"QueryListener.html#child_spec/1","title":"QueryListener.child_spec/1","type":"function"},{"doc":"","ref":"QueryListener.html#consume/3","title":"QueryListener.consume/3","type":"function"},{"doc":"","ref":"QueryListener.html#get_handlers/0","title":"QueryListener.get_handlers/0","type":"function"},{"doc":"","ref":"QueryListener.html#start_link/1","title":"QueryListener.start_link/1","type":"function"},{"doc":"","ref":"RabbitConnection.html","title":"RabbitConnection","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"RabbitConnection.html#child_spec/1","title":"RabbitConnection.child_spec/1","type":"function"},{"doc":"","ref":"RabbitConnection.html#get_connection/1","title":"RabbitConnection.get_connection/1","type":"function"},{"doc":"","ref":"RabbitConnection.html#start_link/1","title":"RabbitConnection.start_link/1","type":"function"},{"doc":"Please see readme for `ReactiveCommons`.","ref":"ReactiveCommons.html","title":"ReactiveCommons","type":"module"},{"doc":"","ref":"ReactiveCommonsSetup.html","title":"ReactiveCommonsSetup","type":"behaviour"},{"doc":"Get config of type AsyncConfig.","ref":"ReactiveCommonsSetup.html#c:config/0","title":"ReactiveCommonsSetup.config/0","type":"callback"},{"doc":"","ref":"ReplyListener.html","title":"ReplyListener","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ReplyListener.html#child_spec/1","title":"ReplyListener.child_spec/1","type":"function"},{"doc":"","ref":"ReplyListener.html#consume/3","title":"ReplyListener.consume/3","type":"function"},{"doc":"","ref":"ReplyListener.html#get_handlers/0","title":"ReplyListener.get_handlers/0","type":"function"},{"doc":"","ref":"ReplyListener.html#start_link/1","title":"ReplyListener.start_link/1","type":"function"},{"doc":"","ref":"ReplyRouter.html","title":"ReplyRouter","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ReplyRouter.html#child_spec/1","title":"ReplyRouter.child_spec/1","type":"function"},{"doc":"","ref":"ReplyRouter.html#delete_reply_route/1","title":"ReplyRouter.delete_reply_route/1","type":"function"},{"doc":"","ref":"ReplyRouter.html#init/1","title":"ReplyRouter.init/1","type":"function"},{"doc":"","ref":"ReplyRouter.html#register_reply_route/2","title":"ReplyRouter.register_reply_route/2","type":"function"},{"doc":"","ref":"ReplyRouter.html#route_reply/2","title":"ReplyRouter.route_reply/2","type":"function"},{"doc":"","ref":"ReplyRouter.html#start_link/1","title":"ReplyRouter.start_link/1","type":"function"},{"doc":"","ref":"SubsConfigProcess.html","title":"SubsConfigProcess","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SubsConfigProcess.html#child_spec/1","title":"SubsConfigProcess.child_spec/1","type":"function"},{"doc":"","ref":"SubsConfigProcess.html#start_link/1","title":"SubsConfigProcess.start_link/1","type":"function"},{"doc":"# ReactiveCommons\n\n[![hex.pm version](https://img.shields.io/hexpm/v/reactive_commons.svg?style=flat)](https://hex.pm/packages/reactive_commons)\n[![hex.pm downloads](https://img.shields.io/hexpm/dt/reactive_commons.svg?style=flat)](https://hex.pm/packages/reactive_commons)\n[![Scorecards supply-chain security](https://github.com/bancolombia/reactive-commons-elixir/actions/workflows/scorecards-analysis.yml/badge.svg)](https://github.com/bancolombia/reactive-commons-elixir/actions/workflows/scorecards-analysis.yml)\n\nThe purpose of `:reactive_commons` is to provide a set of abstractions and implementations over different patterns and\npractices that make the foundation of a reactive microservices' architecture.\n\nEven though the main purpose is to provide such abstractions in a mostly generic way such abstractions would be of\nlittle use without a concrete implementation, so we provide some implementations in a best efforts' manner that aim to\nbe easy to change, personalize and extend.\n\nThe first approach to this work was to release a very simple abstractions, and a corresponding implementation over\nasynchronous message driven communication between microservices build on top of amqp for RabbitMQ.\n\nSee more about this project at [reactivecommons.org](https://reactivecommons.org/)","ref":"readme.html","title":"ReactiveCommons","type":"extras"},{"doc":"#","ref":"readme.html#installation","title":"Installation - ReactiveCommons","type":"extras"},{"doc":"`elixir ~> 1.10`\n\n#","ref":"readme.html#requirements","title":"Requirements - ReactiveCommons","type":"extras"},{"doc":"Releases are published at [Hex](https://hex.pm/packages/reactive_commons), the package can be installed by adding\n`reactive_commons` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:reactive_commons, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#dependencies","title":"Dependencies - ReactiveCommons","type":"extras"},{"doc":"Add `MessageRuntime` to your applications children passing the `AsyncConfig` parameter struct as arguments.\n\n```elixir\n  async_config = AsyncConfig.new(\"my-app-name\")\n  ...\n  children = [\n    {MessageRuntime, async_config},\n  ]\n```","ref":"readme.html#setup","title":"Setup - ReactiveCommons","type":"extras"},{"doc":"There are three semantic structures:\n\n- `DomainEvent`\n  This structure lets you represent an Event in the system. It accepts a `name`, any `data` that will be the information\n  to transport for that event (should be JSON serializable), and an optional `message_id`.\n\n- `Command`\n  Another basic structure is the Command. This structure lets you represent a Command in the system. It accepts a\n  `name`, any `data` that will be the information to transport for that command (should be JSON serializable), and an\n  optional `command_id`.\n\n- `AsyncQuery`\n  Another basic structure is the AsyncQuery. This class lets you represent a Query in the system. It accepts a JSON\n  serializable called `data` that will be the information for that query and a `resource` name for thw query.\n\nThere are three main modules:\n\n- `HandlerRegistry`\n- `DomainEventBus`\n- `DirectAsyncGateway`\n\nThese modules allow the next communication models:\n\n- `DomainEvent` emission through `DomainEventBus` and Event subscription with `HandlerRegistry` from any stakeholder.\n- `Command` emission through `DirectAsyncGateway` to a specific application target and handle this command from the\n  target through `HandlerRegistry`.\n- `AsyncQuery` request through `DirectAsyncGateway` to a specific application target and handle this query from the\n  target through `HandlerRegistry`.","ref":"readme.html#semantic-main-components-definition","title":"Semantic Main Components Definition - ReactiveCommons","type":"extras"},{"doc":"This section describes the reactive API for producing and consuming messages using Reactive Commons\n\n1. Sending Domain Events, Commands and Async Queries\n\n   1.1. Sending Commands\n\n    ```elixir\n    @command_name \"RegisterPerson\"\n    data = Person.new_sample() # any data map\n    ...\n    command = Command.new(@command_name, data)\n    :ok = DirectAsyncGateway.send_command(command, @target) # use any control structure to handle errors\n   ```\n\n   1.2. Sending Async Queries\n\n    ```elixir\n    @query_path \"GetPerson\"\n    data = PersonDataReq.new_sample() # any data map\n    ...\n    query = AsyncQuery.new(@query_path, data)\n    {:ok, person} = DirectAsyncGateway.request_reply_wait(query, @target) # use any control structure to handle errors\n   ```\n\n   1.3. Sending Domain Events\n\n    ```elixir\n    @event_name \"PersonRegistered\"\n    data = PersonRegistered.new_sample() # any data map\n    ...\n    event = DomainEvent.new(@event_name, data)\n    :ok = DomainEventBus.emit(event) # use any control structure to handle errors {:emit_fail, error}\n   ```\n   \n    **See sample project for further details** [Sender](https://github.com/bancolombia/reactive-commons-elixir/blob/main/samples/query-client/lib/query_client/rest_controller.ex)\n\n\n2. Listening and handling for Domain Events, Commands and Async Queries\n\n   ```elixir\n    defmodule QueryServer.SubsConfig do\n        use GenServer\n    \n        @query_name \"GetPerson\"\n        @command_name \"RegisterPerson\"\n        @event_name \"PersonRegistered\"\n    \n        def start_link(_) do\n          GenServer.start_link(__MODULE__, [], name: __MODULE__)\n        end\n    \n        @impl true\n        def init(_) do\n          HandlerRegistry.serve_query(@query_name, &get_person/1) # serve a query, should pass query_name and the function which will handle the request.\n          |> HandlerRegistry.handle_command(@command_name, &register_person/1) # listen for a command, should pass command_name and the function which will handle the command.\n          |> HandlerRegistry.listen_event(@event_name, &person_registered/1) # listen for an event, should pass event_name and the function which will handle the event.\n          |> HandlerRegistry.commit_config() # finally should commit the config to configure the listeners.\n          {:ok, nil}\n        end\n    \n        # Sample functions (should be in a separated module)\n        def get_person(%{} = request) do\n          IO.puts \"Handling async query #{inspect(request)}\"\n          Process.sleep(150)\n          Person.new_sample()\n        end\n    \n        def register_person(%{} = command) do\n          IO.puts \"Handling command #{inspect(command)}\"\n          event = DomainEvent.new(@event_name, PersonRegistered.new_sample(command[\"data\"]))\n          Process.sleep(150)\n          :ok = DomainEventBus.emit(event)\n        end\n    \n        def person_registered(%{} = event) do\n          IO.puts \"Handling event #{inspect(event)}\"\n        end\n    end\n   ```\n\n    **See sample project for further details** [Receiver](https://github.com/bancolombia/reactive-commons-elixir/blob/main/samples/query-server/lib/query_server/subs_config.ex)","ref":"readme.html#usage","title":"Usage - ReactiveCommons","type":"extras"}]}